class Plane { // width:28  height:26
  float moveX;
  float moveY;
  int rotY;
  boolean displayPl = true;
  boolean explodePl = false;

  Plane(float tempX, float tempY, int tempRot) {
    moveX = tempX;
    moveY = tempY;
    rotY = tempRot;
  }

  void display() {
    if (displayPl) {
      pushMatrix();    
      translate(moveX, moveY);
      rotateY(radians(rotY));
      beginShape();
      fill(#ffc545);// yellow
      noStroke();
      rect(0, 10, 4, 8);
      rect(4, 8, 4, 8);
      rect(8, 6, 4, 8);
      rect(12, 0, 4, 26);
      rect(24, 10, 4, 8);
      rect(20, 8, 4, 8);
      rect(16, 6, 4, 8);
      rect(4, 22, 4, 4);
      rect(8, 20, 4, 4);
      rect(16, 20, 4, 4);
      rect(20, 22, 4, 4);
      endShape();
      popMatrix();

      // play flying soundeffect. This repeats the sound without any lag. .loop() has a lag between loops. 
      if (frame%297==0 && (!GameOver && !victory)) {
        file[0].play();
      }
    }
  } //end display

  void explode() {
    displayPl = false; // stop display
    file[0].stop();
    if (explodePl) {       
      pushMatrix();
      translate(moveX, moveY);
      beginShape();
      noStroke();
      fill(#ffc545);
      rect(12, 0, 5, 3);
      rect(16, 3, 5, 3);
      rect(8, 5, 5, 3);
      rect(0, 8, 5, 3);
      rect(16, 8, 5, 3);
      rect(4, 9, 5, 3);
      rect(24, 9, 5, 3);
      rect(12, 12, 5, 3);
      rect(0, 14, 5, 3);
      rect(8, 19, 5, 3);
      rect(12, 21, 5, 3);
      rect(20, 24, 5, 3);
      endShape();
      popMatrix();
    }
  }// end explode

  void moveHor(float tempDirectionX) {
    moveX = lerp(moveX, moveX + tempDirectionX, 0.1); // smoothing the ride..
  }

  void moveVer(float tempDirectionY) {
    if (moveY >= height-60) { // plane can not hit the bottom
      moveY = lerp(moveY, height-60 + tempDirectionY, 0.2);
    } else if (moveY <= height-500) {  // plane can not go higher than 200 pixels
      moveY = lerp(moveY, height-500 + tempDirectionY, 0.2);
    } else {
      moveY = lerp(moveY, moveY + tempDirectionY, 0.1);
    }
  }

  void victory() {
    if (this.moveY <= vertStart+frame) {
      victory = true; // stop game
      frame = 0;
      GameOver = true;      
      stopSounds();
    }
  }

  void victoryDance() {
    if (this.moveX>=width/2) { 
      this.moveY = lerp(this.moveY, this.moveY-20, 0.5);
      this.moveX = lerp(this.moveX, width-wallSize, 0.04);
      this.rotY=15;
    } else {
      this.moveY = lerp(this.moveY, this.moveY-20, 0.5);
      this.moveX = lerp(this.moveX, wallSize, 0.04);
      this.rotY=-15;
    }
  }

  void planeDead() {
    // if plane hits the sidewals it crashes
    if ( this.moveX <= wallSize || this.moveX >= width-wallSize-28 ) {
      this.explodePl = true;
      this.explode();
      file[4].play();
      GameOver = true; // stop game
      gameOver();
    }

    // crashing into Ships
    for (int i=0; i < sh.length; i++) {
      if ( 
        ( (sh[i].direction == 1) && ((this.moveX+28 >= sh[i].moveX && this.moveX <= sh[i].moveX+64) && (this.moveY <= sh[i].moveY+ frame +16 && this.moveY+26 > sh[i].moveY+ frame) && sh[i].displaySh) ) ||
        ( (sh[i].direction == -1) && ((this.moveX+28 >= sh[i].moveX-64 && this.moveX <= sh[i].moveX) && (this.moveY <= sh[i].moveY+frame+16 && this.moveY+26 > sh[i].moveY+frame) && sh[i].displaySh) )
        ) {
        sh[i].explosionTime = 0;
        sh[i].explodeSh = true;
        this.explodePl = true;
        this.explode();
        totScore++; // the score should increment. you did kill a ship, even though it is kamikaze :) 
        file[4].play();
        GameOver = true; // stop game
        gameOver();
      }
    }
    // crashing into Choppers 
    for (int i=0; i < ch.length; i++) {
      if ( 
        ( (ch[i].direction == 1) && ((this.moveX+28 >= ch[i].moveX && this.moveX <= ch[i].moveX+32) && (this.moveY <= ch[i].moveY+ frame +20 && this.moveY+26 > ch[i].moveY+ frame) &&ch[i].displayCh) ) ||
        ( (ch[i].direction == -1) && ((this.moveX+28 >= ch[i].moveX-32 && this.moveX <= ch[i].moveX) && (this.moveY <= ch[i].moveY+frame+20 && this.moveY+26 >ch[i].moveY+frame) && ch[i].displayCh) )
        ) {
        ch[i].explosionTime = 0;
        ch[i].explodeCh = true;
        this.explodePl = true;
        this.explode();
        totScore++; // the score should increment. you did kill a chopper, even though it is kamikaze :)
        file[4].play();
        GameOver = true; // stop game
        gameOver();
      }
    }
  }
}// end class
