
/**
   All code for this project is under the MIT license.
   @author  Aram Pooryousefi
   @version 1.0.0
   @license MIT
**/
import processing.sound.*;
SoundFile[] file = new SoundFile[5];
PImage buttons;
Plane pl;
Chopper[] ch;
Ship[] sh; //array ships..
Jet[] jt;
House[] ho;
Tree[] tr;
ArrayList<Bullet> bu;

boolean shooting, toRight, toLeft, toUp, toDown, GameOver, bulletAlive, victory, timeIsOver; 
int frame, totBullet, totScore, gameDifficulty, gameDuration, vertStart, iniShip, iniChopper, iniJet;
float bulletTimer, wallSize;
String[] direction = {"left", "right"};

/*
 ____________________________________________________________
 |                                                           |
 |   Creating Games setup:                                   |
 |   changing numbers changes the game length, speed, etc.. |
 |___________________________________________________________|
 */
void gameSetup() {
  gameDuration = 60;    // timer of the game in sec.
  gameDifficulty = 3;   // choose between 1 (easy) and 5(hard)
  vertStart = -3000;      // the physical length of the game
  iniShip = abs(vertStart)/100;
  iniChopper= abs(vertStart)/100;
  iniJet= abs(vertStart)/200;
  
  ho= new House[abs(vertStart)/100];
  tr= new Tree[abs(vertStart)/180];

  // Do not change the numbers below!
  ch = new Chopper[gameDifficulty + iniChopper];
  sh = new Ship[iniShip + gameDifficulty]; //array ships..
  jt = new Jet[iniJet + gameDifficulty];
  wallSize = width/4;
}

void setup() {
  loadSounds();
  size(900, 800, P3D);
  buttons = loadImage("buttons.png");
  frameRate(60);
  gameSetup();
  startGame();
  // uncomment this line below to start the game after pressing R, otherwise game starts immediately
  gameOver();
}

void draw() {  
  background(#584fda);
  leftWall();
  rightWall();
  pl.display();
  victoryZone();
  displayShips();
  displayChoppers();
  //displayJets();  // >> not completed yet..
  displayHouses();
  displayTrees();
  gameTimer(gameDuration);

  if (GameOver == false) {              // as long as the game is alive
    pl.planeDead();
    shootBullets();
    handleKeyPresses();
    if (victory==false) {
      frame++;                          // only count when the game is still going && not won yet.
    }
    bulletTimer = bulletTimer + 0.25;   // the time needed before new bullet is shot
  } else {
    gameOver();
  }
  pl.victory();                         // check if player has won
  // when the game is won, the plane flys away
  if (victory==true) {
    pl.victoryDance();
  }

  if (frame >= gameDuration*60) {    
    timeIsOver=true;
    gameOver();
  }
}// end draw

/*
 ____________________________
 |                           |
 |   Creating the walls      |
 |___________________________|
 */
void leftWall() {
  pushMatrix();
  fill(#649228);
  rect(0, 0, wallSize, height);
  popMatrix();
}
void rightWall() {
  pushMatrix();
  fill(#649228);
  rect(width-wallSize, 0, width, height);
  popMatrix();
}

/*
 ____________________________
 |                           |
 |   Displaying all objects  |
 |___________________________|
 */
void displayShips() { // displaying all the ships
  for (int i=0; i < sh.length; i++) {    
    sh[i].explodeTimer();
    sh[i].display();
    sh[i].move();
    if (sh[i].explosionTime <15) {
      sh[i].explode();
    }
  }
}
void displayChoppers() { //displaying all the choppers
  for (int i=0; i < ch.length; i++) {
    ch[i].explodeTimer();
    ch[i].display();
    ch[i].move();
    if (ch[i].explosionTime <15) {
      ch[i].explode();
    }
  }
}
void displayJets() { //displaying all the Jets
  for (int i=0; i < jt.length; i++) {
    jt[i].explodeTimer();
    jt[i].display();
    jt[i].move();
    if (jt[i].explosionTime <15) {
      jt[i].explode();
    }
  }
}
void displayHouses() { // displaying all the Houses
  for (int i=0; i < ho.length; i++) {    
    ho[i].display();
  }
}
void displayTrees() { // displaying all trees
  for (int i=0; i < tr.length; i++) {    
    tr[i].display();
  }
}

/*
 _______________________________
 |                              |
 |   Handeling the keyspresses  |
 |______________________________|
 */
void handleKeyPresses() {
  //shooting: every click cretes a new bullets.
  if ( shooting == true) {
    if (bulletTimer%2==0) {
      bulletAlive = true;
      bu.add(new Bullet());
      file[1].play();
    }
  }
  if (toRight || toLeft) {
    if (toRight == true) {
      pl.moveHor(50);
      pl.rotY=10;
    } 
    if (toLeft == true) {
      pl.moveHor(-50);
      pl.rotY=-10;
    }
  } else {
    pl.rotY=0;
  }
  if (toUp == true) {
    pl.moveVer(-20);
  }
  if (toDown == true) {
    pl.moveVer(20);
  }
}

void keyPressed() {
  if (key == 'r' || key == 'R') {
    restart();
  }
  if (key == ' ') {
    shooting = true;
  }
  if (key == 'a' || key == 'A' || keyCode==LEFT) {
    toLeft = true;
    toRight = false;
    toUp = false;
    toDown = false;
  }
  if (key == 'd' || key == 'D' || keyCode==RIGHT) {
    toRight = true;
    toLeft = false;
    toUp = false;
    toDown = false;
  }
  if (key == 'w' || key == 'W' || keyCode==UP) {
    toUp = true;
    toLeft = false;
    toRight = false;
    toDown = false;
  }
  if (key == 's' || key == 'S' || keyCode==DOWN) {
    toDown = true;
    toLeft = false;
    toRight = false;
    toUp = false;
  }
}
void keyReleased() {
  if (key == 'a' || key == 'A' || keyCode==LEFT) {
    toLeft = false;
  }
  if (key == 'd' || key == 'D' || keyCode==RIGHT) {
    toRight = false;
  }
  if (key == 'w' || key == 'W' || keyCode==UP) {
    toUp = false;
  }
  if (key == 's' || key == 'S' || keyCode==DOWN) {
    toDown = false;
  }
  if (key == ' ') {
    shooting = false;
  }
}

// shoot with mouseClick
void mousePressed() {
  bulletAlive = true;
  bu.add(new Bullet());
  if (!GameOver) {
    file[1].play();
  }
}

/*
 ________________________________________
 |                                       |
 |   Starting the game/defining objects  |
 |_______________________________________|
 */
void startGame() {
  int heightActiveShip= (abs(vertStart)+height-250)/sh.length;    // this is needed so ships will not be created on the same height as much as pssible.
  int heightActiveChopper= (abs(vertStart)+height-250)/ch.length; // same as above for choppers
  int heightActiveJet= (abs(vertStart)+height-250)/jt.length;
  int heightActiveHouse= (abs(vertStart)+height-100)/(ho.length/2); // same as above for choppers
  // Creating Ships
  for (int i=0; i < sh.length; i++) {
    sh[i] = new Ship(int(random(wallSize + 0, width-wallSize-64)), int(random(vertStart + heightActiveShip*i, vertStart +(heightActiveShip*(i+1)))), direction[int(random(direction.length))] );
  }
  // Creating Choppers
  for (int i=0; i < ch.length; i++) {
    ch[i] = new Chopper(int(random(wallSize + 0, width-wallSize -32)), int(random(vertStart + heightActiveChopper*i, vertStart +(heightActiveChopper*(i+1)))), direction[int(random(direction.length))] );
  }
  // Creating Jets
  for (int i=0; i < jt.length; i++) {
    jt[i] = new Jet(int(random(wallSize + 0, width-wallSize -32)), int(random(vertStart + heightActiveJet*i, vertStart +(heightActiveJet*(i+1)))), direction[int(random(direction.length))] );
  }
  // creating hoses
  for (int i=0; i < ho.length; i++) {
    if (i < ho.length/2) {
      // creating houses for the left side
      ho[i] = new House(int(random(15, wallSize-60)), int(random(vertStart + heightActiveHouse*i, vertStart +(heightActiveHouse*(i+1)))));
    } else { 
      // creating houses for the right side
      ho[i] = new House(int(random(width-wallSize+15, width-60)), int(random(vertStart + heightActiveHouse*(i-ho.length/2), vertStart +(heightActiveHouse*((i-ho.length/2)+1)))));
    }
  }  
  //creating all the trees
  for (int i=0; i < tr.length; i++) {
    if (i < tr.length/2) {
      // creating trees for the left side
      tr[i] = new Tree(int(random(15, wallSize-60)), int(random(0, height-50)));
    } else { 
      // creating trees for the right side
      tr[i] = new Tree(int(random(width-wallSize+15, width-60)), int(random(0, height-50)));
    }
  }
  // Creating plane 
  pl = new Plane(width/2, height-60, 0);
  // bullets: creating arraylist of bullets
  bu = new ArrayList<Bullet>();
  for (int i = 0; i < totBullet; i++) {
    bu.add(new Bullet());
  }
}

/*
 ______________________________________________
 |                                             |
 |   Handeling the bullets: create and remove  |
 |_____________________________________________|
 */
void shootBullets() {
  for (int i=0; i <  bu.size(); i++) {
    Bullet b = bu.get(i);
    b.display();
    totBullet++;
    if (b.bulletDead()) {
      bu.remove(i);
    }
  }
}

/*
 ______________________________________________
 |                                             |
 |   Loading/stopping different soudfiles      |
 |_____________________________________________|
 */
void loadSounds() {
  file[0] = new SoundFile (this, "jet.mp3");
  file[1] = new SoundFile (this, "shoot.mp3");
  file[2] = new SoundFile (this, "explodeplane.mp3");
  file[3] = new SoundFile (this, "explodeplane.mp3");
  file[4] = new SoundFile (this, "explodeplane.mp3");
  // adjusting the volume of each sound
  file[0].amp(0.5);
  file[2].amp(0.5);
  file[3].amp(0.5);
  file[4].amp(1);
}

void stopSounds() {
  file[0].stop();
  file[1].stop();
  file[2].stop();
  file[3].stop();
  file[4].stop();
}

/*
 ______________________________________________
 |                                             |
 |   Timer of the game                         |
 |_____________________________________________|
 */
void gameTimer(float tempDurationGame) {
  if (GameOver==false) {
    float xPos = width - ((width * frame) / (tempDurationGame *60));
    // the timer bar changes color to indicate the urge of finishing. 
    if (xPos >= width/2) {
      fill(#00c717);// groen
    } else if (xPos < width/2 && xPos >=width/4) {
      fill(#ffc545);// geel
    } else {
      fill(#ff270a); // rood
    }
    noStroke();
    rect(0, 0, xPos, 10);
  }
}

/*
 ______________________________________________
 |                                             |
 |   Cearing the save zone for the game        |
 |_____________________________________________|
 */
void victoryZone() {
  if (GameOver==false) {
    //create victory zone: the end line everything under this line moves downwards
    strokeWeight(1);
    stroke(#ffffff);
    pushMatrix();
    translate(0, vertStart+frame);
    line(0, 0, wallSize, 0);
    line(width-wallSize, 0, width, 0);
    popMatrix();

    // text above the line
    pushMatrix();
    translate(0, vertStart-60+frame);
    textSize(20);
    fill(255);
    textAlign(CENTER);
    text("Reach in time to win", 0, 30, width, 30);
    popMatrix();
  }
}

/*
 ______________________________________________
 |                                             |
 |   Cearing the game over function            |
 |_____________________________________________|
 */
void gameOver() {
  if (victory==false) {
    GameOver = true;
    if (frame >0) {
      fill(#584fda);
      noStroke();
      rect(0, 0, width, height);
      translate(0, 0);

      textSize(25);
      fill(255);
      textAlign(CENTER);
      if (timeIsOver) {
        text("GAME OVER: Time is up!", 0, height/2, width, height/2);
        stopSounds();
      } else {
        text("GAMEOVER", 0, height/2, width, height/2);
      }
      textSize(30);
      text("Total score: " + totScore, 0, height/2-50, width, height/2-50);
      pl.explode();
    }
  }
  if (victory == true) {
    textSize(25);
    fill(255);
    textAlign(CENTER);
    text("You're save!", 0, height/2, width, height/2);
    textSize(30);
    text("Total score: " + totScore, 0, height/2-50, width, height/2-50);
    stopSounds();
  }

  // show the instruction image
  imageMode(CENTER);
  image(buttons, width/2, height*1/5);

  // after gameover still show the walls
  leftWall();
  rightWall();
}

/*
 ______________________________________________
 |                                             |
 |   Cearing restart function for the game     |
 |_____________________________________________|
 */
void restart() {
  if (GameOver==true) {
    GameOver=false;
    victory = false;
    timeIsOver = false;
    totScore =0;
    totBullet=0;
    frame=0;
    bulletTimer=0;
    //vertStart =-10;

    stopSounds();
    startGame();
    gameTimer(gameDuration);
  }
}
